package dnd;

import java.awt.Component;
import java.awt.Composite;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

public class DnDGlassPane extends JPanel {

  private static final long serialVersionUID = 1L;

  private final static String KEY_TRANSPARENCY = ".transparency";
  private final static Image DEFAULT_IMAGE = UIHelper.readImage(
      DnDGlassPane.class, "images/default.png");

  protected RolloverImageBuilder rolloverImageBuilder;
  protected Properties properties;

  protected final Set<DragItem> dragItems = new HashSet<DragItem>();
  protected final Timer timer = new Timer(500, new ActionListener() {
    @Override
    public void actionPerformed(ActionEvent e) {
      showing = !showing;
      repaint();
    }
  });

  protected Component rollover;
  protected Image rolloverImage;
  protected Point rolloverLocation;

  protected Dimension drag;
  protected boolean showing;

  public DnDGlassPane(RolloverImageBuilder rolloverImageBuilder) {
    this(rolloverImageBuilder, null);
  }

  public DnDGlassPane(RolloverImageBuilder rolloverImageBuilder,
      Properties properties) {
    setOpaque(false);
    this.rolloverImageBuilder = rolloverImageBuilder;
    this.properties = properties;
    timer.setRepeats(true);
  }

  public RolloverImageBuilder getRolloverImageBuilder() {
    return rolloverImageBuilder;
  }

  public void setRolloverImageBuilder(RolloverImageBuilder rolloverImageBuilder) {
    this.rolloverImageBuilder = rolloverImageBuilder;
  }

  public boolean addDragItem(DragItem dragged) {
    boolean canAdd = dragItems.isEmpty()
        || (compliantDRAG_EXCLUSIVEFlag(dragged) && compliantDRAG_GROUP_ENABLEDFlag(dragged));

    if (canAdd) {
      showing = true;
      dragItems.add(dragged);
      if (dragItems.size() == 1) {
        setVisible(true);
        timer.start();
      } else {
        repaint();
      }
    }

    return canAdd;
  }

  private boolean compliantDRAG_EXCLUSIVEFlag(DragItem dragged) {
    return (dragItems.iterator().next().getDragFlags() & DragItem.DRAG_EXCLUSIVE) == 0
        && (dragged.getDragFlags() & DragItem.DRAG_EXCLUSIVE) == 0;
  }

  private boolean compliantDRAG_GROUP_ENABLEDFlag(DragItem dragged) {
    int enabled = dragged.getDragFlags() & DragItem.DRAG_GROUP_ENABLED;
    if ((dragItems.iterator().next().getDragFlags() & DragItem.DRAG_GROUP_ENABLED) != enabled)
      return false;
    else if (enabled == 0)
      return true;
    else if (dragged.getGroups() == null || dragged.getGroups().isEmpty())
      return false;

    Set<DragGroup> commonGroups = new HashSet<DragGroup>(dragged.getGroups());
    for (DragItem item : dragItems) {
      commonGroups.retainAll(item.getGroups());
      if (commonGroups.isEmpty()) {
        return false;
      }
    }
    return true;
  }

  public void removeDragItem(DragItem dragged) {
    showing = true;
    dragItems.remove(dragged);
    if (dragItems.isEmpty()) {
      dispose();
    } else {
      repaint();
    }
  }

  public void dragStarted() {
    timer.stop();
    setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
  }

  public void dispose() {
    dragItems.clear();
    timer.stop();
    setCursor(Cursor.getDefaultCursor());
    rollover = null;
    rolloverImage = null;
    rolloverLocation = null;
    drag = null;
    showing = false;
    setVisible(false);
  }

  public Component getRollover() {
    return rollover;
  }

  public void useDropListener(DropListener dropListener) {
    Component rollover = getRollover(dropListener);

    if (rollover != this.rollover) {
      this.rollover = rollover;

      if (rollover == null) {
        rolloverImage = null;
        rolloverLocation = null;
      } else {
        RolloverImageBuilder rolloverImageBuilder = getRolloverImageBuilder(dropListener);
        rolloverImage = rolloverImageBuilder.createRolloverImage(rollover);
        rolloverLocation = getRolloverLocation(rollover);
      }
    }
  }

  protected Component getRollover(DropListener dropListener) {
    Component rollover = dropListener != null ? dropListener.getDropTarget()
        : null;
    return rollover;
  }

  protected RolloverImageBuilder getRolloverImageBuilder(
      DropListener dropListener) {
    RolloverImageBuilder rolloverImageBuilder = dropListener != null
        && dropListener.getRolloverImageBuilder() != null ? dropListener
        .getRolloverImageBuilder() : this.getRolloverImageBuilder();
    return rolloverImageBuilder;
  }

  protected Point getRolloverLocation(Component rollover) {
    Point rolloverLocation = rollover.getLocationOnScreen();
    SwingUtilities.convertPointFromScreen(rolloverLocation, this);
    return rolloverLocation;
  }

  public void setDrag(int dx, int dy) {
    if (drag == null) {
      drag = new Dimension();
    }
    drag.setSize(dx, dy);
  }

  @Override
  protected void paintComponent(Graphics g) {
    if (dragItems.isEmpty() || (drag == null && !showing))
      return;

    Graphics2D g2 = (Graphics2D) g;
    if (rolloverImage != null) {
      g2.drawImage(rolloverImage, rolloverLocation.x, rolloverLocation.y, this);
    }

    for (DragItem dragged : dragItems) {
      Point location = dragged.getGlassPaneLocation();

      if (location != null) {
        g2.setComposite(getComposite(dragged));

        Image image = dragged.getImage() != null ? dragged.getImage()
            : DEFAULT_IMAGE;
        int x0 = location.x - image.getWidth(this) / 2;
        int y0 = location.y - image.getHeight(this) / 2;
        g2.drawImage(image, x0 + (drag != null ? drag.width : 0), y0
            + (drag != null ? drag.height : 0), this);
      }
    }
  }

  protected Composite getComposite(DragItem dragged) {
    String property = null;

    if (properties != null) {
      String keyTransparency = dragged.getClass().getName() + KEY_TRANSPARENCY;

      if (dragged instanceof DragPicture) {
        DragPicture dragPicture = (DragPicture) dragged;

        if (dragPicture.getFilename() != null) {
          property = properties.getProperty(keyTransparency + "."
              + dragPicture.getFilename());
        }
      }

      if (property == null) {
        property = properties.getProperty(keyTransparency);
      }
    }

    return property != null ? UIHelper.getAlphaCompositeInstance(Float
        .parseFloat(property)) : UIHelper.transparency50;
  }
}