package dnd;

import static dnd.DropListener.SUPPORTS_COLLAPSED_EVENT;
import static dnd.DropListener.SUPPORTS_MULTI_SOURCE;
import static dnd.DropListener.SUPPORTS_MULTI_SOURCE_WITH_UNLISTENED_SOURCE;

import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JRootPane;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;

public class DnDManager extends MouseInputAdapter implements
    WindowFocusListener, HierarchyListener {

  protected final DnDGlassPane glassPane;

  protected final Map<Component, DragItem> map = new HashMap<Component, DragItem>();

  protected final Collection<Component> selections = new LinkedHashSet<Component>();

  protected Point dragLocation;
  protected DropListener dropListener;

  private boolean windowLostFocusEventEnabled = true;
  private boolean dndEnabled;

  public DnDManager(final DnDGlassPane glass) {
    glassPane = glass;
    checkGlassPane();
    glassPane.addHierarchyListener(this);
    addWindowFocusListener(glassPane);
  }

  private void checkGlassPane() {
    if (glassPane == null) {
      throw new NullPointerException("glassPane cannot be set to null.");
    }
    Container container = glassPane.getParent();
    if (container != null) {
      JRootPane rootPane = container instanceof JRootPane ? (JRootPane) container
          : null;
      if (rootPane == null || rootPane.getGlassPane() != glassPane) {
        throw new IllegalArgumentException(
            "glassPane.parent is not a JRootPane.");
      }
    }
  }

  public void register(Component c, DragItem item) {
    DragItem dragged = null;

    if (c != null && item != null && (dragged = map.put(c, item)) == null) {
      c.addMouseListener(this);
      c.addMouseMotionListener(this);
    }

    if (dragged != null && dragged != item && selections.contains(c)) {
      item.updateImage(c);
      item.setGlassPaneLocation(dragged.getGlassPaneLocation());
      glassPane.addDragItem(item);
      glassPane.removeDragItem(dragged);
    }
  }

  public void unregister(Component c) {
    DragItem dragged = null;

    if (c != null && (dragged = map.remove(c)) != null) {
      c.removeMouseListener(this);
      c.removeMouseMotionListener(this);

      if (selections.remove(c)) {
        glassPane.removeDragItem(dragged);
      }
    }
  }

  @Override
  public void mousePressed(MouseEvent e) {
    if (!dndEnabled)
      return;

    Component c = e.getComponent();
    DragItem dragged = map.get(c);

    if (dragged != null) {
      if (selections.remove(c)) {
        glassPane.removeDragItem(dragged);
      } else if (glassPane.addDragItem(dragged)) {
        selections.add(c);
        dragged.updateImage(c);
        dragged.setGlassPaneLocation(getGlassPaneLocation(e));
      }
    }
  }

  @Override
  public void mouseDragged(MouseEvent e) {
    if (!dndEnabled)
      return;

    Point location = getGlassPaneLocation(e);

    if (dragLocation == null) {
      dragLocation = location;
      glassPane.dragStarted();
    }

    dropListener = getDropListener(e.getComponent(), location);
    glassPane.useDropListener(dropListener);
    glassPane.setDrag(location.x - dragLocation.x, location.y - dragLocation.y);
    glassPane.repaint();
  }

  @Override
  public void mouseReleased(MouseEvent e) {
    if (!dndEnabled)
      return;

    Component rollover = glassPane.getRollover();

    if (rollover != null) {
      // Ici, on doit ignorer les vnements WINDOW_LOST_FOCUS car il est
      // possible de perdre le focus lors de l'excution d'un callback
      // 'dropPerformed', ce qui aurait pour effet de dclencher la mthode
      // 'windowLostFocus' -> 'dispose' -> 'selections.clear' alors qu'on
      // parcourt 'selections' => exception ConcurrentModificationException !
      windowLostFocusEventEnabled = false;

      Point dropLocation = UIHelper.getTargetLocation(e, rollover);

      for (Component dragSource : selections) {
        DragItem dragged = map.get(dragSource);

        dropListener.dropPerformed(new DropEvent(dragSource, dragged
            .getAction(), rollover, dropLocation));
      }

      if ((dropListener.getSupportsFlags() & SUPPORTS_COLLAPSED_EVENT) != 0) {
        dropListener.dropPerformed(new DropCollapsedEvent(selections, rollover,
            dropLocation));
      }

      // On permet  nouveau la prise en compte des vnements WINDOW_LOST_FOCUS
      windowLostFocusEventEnabled = true;
    }

    if (dragLocation != null) {
      dispose();
    }
  }

  protected Point getGlassPaneLocation(MouseEvent e) {
    Point location = UIHelper.getTargetLocation(e, glassPane);
    return location;
  }

  protected DropListener getDropListener(Component c, Point location) {
    JRootPane rootPane = SwingUtilities.getRootPane(c);
    Component rollover;

    if (rootPane == null
        || (rollover = SwingUtilities.getDeepestComponentAt(rootPane
            .getContentPane(), location.x, location.y)) == null) {
      return null;
    }

    if (dropListener != null && dropListener.getDropTarget() == rollover) {
      return dropListener;
    }

    boolean monoSource = selections.size() == 1;
    Set<DropListener> sharedListeners = null;

    for (Component dragSource : selections) {
      DragItem dragged = map.get(dragSource);
      List<DropListener> listenersNotSupportingUnlistenedSource = null;

      for (DropListener listener : dragged.getDropListeners()) {
        if (rollover == listener.getDropTarget()
            && (monoSource || (listener.getSupportsFlags() & SUPPORTS_MULTI_SOURCE) != 0)) {

          if (monoSource
              || (listener.getSupportsFlags() & SUPPORTS_MULTI_SOURCE_WITH_UNLISTENED_SOURCE) != 0) {
            return listener;
          }

          if (listenersNotSupportingUnlistenedSource == null) {
            listenersNotSupportingUnlistenedSource = new ArrayList<DropListener>();
          }
          listenersNotSupportingUnlistenedSource.add(listener);
        }
      }

      if (listenersNotSupportingUnlistenedSource == null) {
        return null;
      }

      if (sharedListeners == null) {
        sharedListeners = new HashSet<DropListener>(
            listenersNotSupportingUnlistenedSource);
      } else {
        sharedListeners.retainAll(listenersNotSupportingUnlistenedSource);
      }
    }

    return sharedListeners == null || sharedListeners.isEmpty() ? null
        : sharedListeners.iterator().next();
  }

  protected void dispose() {
    selections.clear();
    dragLocation = null;
    dropListener = null;
    glassPane.dispose();
  }

  @Override
  public void windowGainedFocus(WindowEvent e) {
  }

  @Override
  public void windowLostFocus(WindowEvent e) {
    // 'dispose' si et seulement si on permet la prise en compte des vnements
    // WINDOW_LOST_FOCUS
    if (windowLostFocusEventEnabled) {
      dispose();
    }
  }

  @Override
  public void hierarchyChanged(HierarchyEvent e) {
    if ((e.getChangeFlags() & HierarchyEvent.PARENT_CHANGED) != 0
        && e.getChanged() == glassPane) {

      if (glassPane.getParent() == null) {
        // glassPane was removed => find the old glassPane parent window to
        // remove its window focus listener and disable drag and drop
        removeWindowFocusListener(e.getChangedParent());
      } else {
        // check that glassPane has been added to a RootPaneContainer
        checkGlassPane();
        // glassPane was added => find the new glassPane parent window to
        // add its window focus listener and enable drag and drop
        addWindowFocusListener(e.getChangedParent());
      }
    }
  }

  private void addWindowFocusListener(Container c) {
    Window w = SwingUtilities.getWindowAncestor(c);
    if (w != null) {
      w.addWindowFocusListener(this);
      dndEnabled = true;
    }
  }

  private void removeWindowFocusListener(Container c) {
    Window w = SwingUtilities.getWindowAncestor(c);
    if (w != null) {
      w.removeWindowFocusListener(this);
    }
    dndEnabled = false;
    dispose();
  }
}