package dnd;

import static dnd.ColoredShape.createRectangle;

import java.awt.Color;
import java.awt.Component;
import java.awt.Image;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;

public interface RolloverImageBuilder {

  Image createRolloverImage(Component rollover);

  RolloverImageBuilder rec0 = new RecImageBuilder(true, Color.black, false);
  RolloverImageBuilder rec1 = new RecImageBuilder(true, Color.darkGray);

  RolloverImageBuilder roundRec0 = new RoundRecImageBuilder(false, Color.red);
  RolloverImageBuilder roundRec1 = new RoundRecImageBuilder(true, Color.red);

  RolloverImageBuilder ellipse0 = new EllipseImageBuilder(false, Color.green);
  RolloverImageBuilder ellipse1 = new EllipseImageBuilder(true, Color.green);

  RolloverImageBuilder negative15 = new NegativeImageBuilder(15);

  RolloverImageBuilder raisedEtched = new BorderedImageBuilder(BorderFactory
      .createEtchedBorder(EtchedBorder.RAISED));
  RolloverImageBuilder loweredEtched = new BorderedImageBuilder(BorderFactory
      .createEtchedBorder(EtchedBorder.LOWERED));
  RolloverImageBuilder raisedBevel = new BorderedImageBuilder(BorderFactory
      .createRaisedBevelBorder());
  RolloverImageBuilder loweredBevel = new BorderedImageBuilder(BorderFactory
      .createLoweredBevelBorder());

  abstract class ShapeImageBuilder implements RolloverImageBuilder {

    protected boolean additive;
    protected Color color;
    protected boolean filled;

    public ShapeImageBuilder(boolean additive, Color color) {
      this(additive, color, true);
    }

    public ShapeImageBuilder(boolean additive, Color color, boolean filled) {
      this.additive = additive;
      this.color = color;
      this.filled = filled;
    }

    @Override
    public Image createRolloverImage(Component rollover) {
      ColoredShape shape = createShape(rollover, filled);
      return UIHelper.createShapeImage(rollover.getWidth(), rollover
          .getHeight(), additive, shape);
    }

    protected abstract ColoredShape createShape(Component rollover,
        boolean filled);
  }

  class RecImageBuilder extends ShapeImageBuilder {

    public RecImageBuilder(boolean additive, Color color) {
      super(additive, color);
    }

    public RecImageBuilder(boolean additive, Color color, boolean filled) {
      super(additive, color, filled);
    }

    @Override
    public ColoredShape createShape(Component rollover, boolean filled) {
      ColoredShape shape = createRectangle(0, 0, rollover.getWidth(), rollover
          .getHeight(), color, filled);
      return shape;
    }
  }

  class RoundRecImageBuilder extends ShapeImageBuilder {

    protected int size;

    public RoundRecImageBuilder(boolean additive, Color color) {
      this(additive, color, 20, true);
    }

    public RoundRecImageBuilder(boolean additive, Color color, int size) {
      this(additive, color, size, true);
    }

    public RoundRecImageBuilder(boolean additive, Color color, int size,
        boolean filled) {
      super(additive, color, filled);
      this.size = size;
    }

    @Override
    public ColoredShape createShape(Component rollover, boolean filled) {
      ColoredShape shape = ColoredShape.createRoundRectangle(0, 0, rollover
          .getWidth(), rollover.getHeight(), size, size, color, filled);
      return shape;
    }
  }

  class EllipseImageBuilder extends ShapeImageBuilder {

    public EllipseImageBuilder(boolean additive, Color color) {
      super(additive, color);
    }

    public EllipseImageBuilder(boolean additive, Color color, boolean filled) {
      super(additive, color, filled);
    }

    @Override
    public ColoredShape createShape(Component rollover, boolean filled) {
      ColoredShape shape = ColoredShape.createEllipse(0, 0,
          rollover.getWidth(), rollover.getHeight(), color, filled);
      return shape;
    }
  }

  class NegativeImageBuilder implements RolloverImageBuilder {

    protected int brightness;

    public NegativeImageBuilder(int brightness) {
      this.brightness = brightness;
    }

    @Override
    public Image createRolloverImage(Component rollover) {
      return UIHelper.createFilteredImage(UIHelper.createImage(rollover),
          new NegativeFilter(brightness));
    }
  }

  class BorderedImageBuilder implements RolloverImageBuilder {

    protected Border border;
    protected Color bg;

    public BorderedImageBuilder(Border border) {
      this.border = border;
    }

    public BorderedImageBuilder(Border border, Color bg) {
      this.border = border;
      this.bg = new Color((bg.getRGB() & 0x00ffffff) | 0x30000000, true);
    }

    @Override
    public Image createRolloverImage(Component rollover) {
      JPanel renderer = new JPanel();
      renderer.setOpaque(bg != null);
      renderer.setFont(rollover.getFont());
      renderer.setBounds(rollover.getBounds());
      renderer.setForeground(rollover.getForeground());
      renderer.setBackground(bg != null ? bg : rollover.getBackground());
      renderer.setBorder(border);
      return UIHelper.createImage(renderer);
    }
  }
}
