package dnd;

import java.awt.AlphaComposite;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

public final class UIHelper {

  private final static RenderingHints hints = new RenderingHints(
      RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
  static {
    hints.put(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);
  }

  private final static Map<Float, AlphaComposite> mapAlphaComposites = new HashMap<Float, AlphaComposite>();

  public final static AlphaComposite transparency25 = getAlphaCompositeInstance(0.25f);
  public final static AlphaComposite transparency50 = getAlphaCompositeInstance(0.5f);

  public static AlphaComposite getAlphaCompositeInstance(float alpha) {
    AlphaComposite alphaComposite = mapAlphaComposites.get(alpha);
    if (alphaComposite == null) {
      alphaComposite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
          alpha);
      mapAlphaComposites.put(alpha, alphaComposite);
    }
    return alphaComposite;
  }

  public static JLabel createLabel(Class<?> cl, String filename) {
    ImageIcon image = readImageIcon(cl, filename);
    JLabel label = new JLabel(image);
    return label;
  }

  public static JLabel createLabel(Class<?> cl, String text, String filename) {
    ImageIcon image = readImageIcon(cl, filename);
    JLabel label = new JLabel(text, image, SwingConstants.CENTER);
    label.setHorizontalTextPosition(SwingConstants.CENTER);
    label.setVerticalTextPosition(SwingConstants.BOTTOM);
    return label;
  }

  public static ImageIcon readImageIcon(Class<?> cl, String filename) {
    return new ImageIcon(readImage(cl, filename));
  }

  public static Image readImage(Class<?> cl, String filename) {
    return Toolkit.getDefaultToolkit().getImage(cl.getResource(filename));
  }

  public static Image createImage(Component c) {
    int width = c.getWidth();
    int height = c.getHeight();

    BufferedImage image = new BufferedImage(width, height,
        BufferedImage.TYPE_INT_ARGB);
    Graphics g = image.getGraphics();
    c.paint(g);
    g.dispose();

    return image;
  }

  public static Image createShapeImage(int width, int height, boolean additive,
      ColoredShape... shapes) {

    BufferedImage image = new BufferedImage(width, height,
        BufferedImage.TYPE_INT_ARGB);
    Graphics2D g2 = (Graphics2D) image.createGraphics();

    if (shapes.length > 0) {
      g2.setRenderingHints(hints);
      Composite composite = g2.getComposite();

      ColoredShape filler = additive ? null : ColoredShape.createRectangle(0,
          0, width, height, null, true);

      for (ColoredShape shape : shapes) {
        Area mask;

        if (additive) {
          mask = new Area(shape.getShape());
        } else {
          mask = new Area(filler.getShape());
          mask.subtract(new Area(shape.getShape()));
        }

        g2.setColor(shape.getColor());
        if (shape.isFilled()) {
          g2.setComposite(transparency25);
          g2.fill(mask);
          g2.setComposite(composite);
        }
        g2.draw(mask);
      }
    }
    g2.dispose();

    return image;
  }

  public static Image createFilteredImage(Image image, ImageFilter filter) {
    ImageProducer p = new FilteredImageSource(image.getSource(), filter);
    Image filteredImage = Toolkit.getDefaultToolkit().createImage(p);
    return filteredImage;
  }

  public static Point getTargetLocation(MouseEvent e, Component target) {
    Point location = e.getPoint();
    SwingUtilities.convertPointToScreen(location, e.getComponent());
    SwingUtilities.convertPointFromScreen(location, target);
    return location;
  }
}
